#include <I2CMiniTerminal.h>
#include <Wire.h>
#include <TimerOne.h> // Timer 1 library
#include <dht22.h> // Library from Virtuabotix

// instantiate two sensor classes
dht22 sensor0;
dht22 sensor1;

#define DEBUG

int8_t setTempHigh = -1;
int8_t setTempLow = -1;
int8_t setHumidity = -1;
int8_t tempOutdoor = -1;
int8_t tempIndoor = -1;
int8_t humidityOutdoor = -1;
int8_t humidityIndoor = -1;

uint8_t fanState = 0;

uint8_t idleState = 0;
#define MAXIDLE = 6;

// Store strings in program memory
p_string(tempOut) = "Temp Out";
p_string(tempIn) = "Temp In";
p_string(humOut) = "Hum Out";
p_string(humIn) = "Hum In";
p_string(tempHighSet) = "TH Set:";
p_string(tempLowSet) = "TL Set:";
p_string(humiditySet) = "H Set:";

p_string(progname) = "FAC TEST";
p_string(progrev) = "Rev 004";

uint8_t gotNewKey = 0; // set to 1 if a new key is available
uint8_t checkKey = 0; // set to 1 by timer interrupt
uint8_t newKey = 0; // variable for newKey

int fanPin = 12;

void setup()
{
  #ifdef DEBUG
  Serial.begin(57600);
  Serial.println("Fresh Air Controller 0.04");
  #endif  
  
  // initialize the digital pin as an output.
  pinMode(fanPin, OUTPUT);
  
  // Uses DHT.h to attach two DHT22 Sensors
  sensor0.attach(10);
  sensor1.attach(11);
  
  Wire.begin(); // join i2c bus (address optional for master)

  // initialize timer1
  Timer1.initialize(100000); // trips 10 times a second
  Timer1.attachInterrupt(checkKeyTimer); 

  mt_clear(); // Clear the LCD 
  
  mt_printPM(progname,0,0);
  mt_printPM(progrev,0,1); 
  delay(2000); // So you can view the progname and progrev
 
  // Start clear and zero'd cursor
  mt_clear();
  mt_cursor();
  mt_setCursor(0,0); 
  
}

void loop()
{

 // We will run the idle() routine until we receive
 // a CTR key, then we will run the menu() routine
 // until it returns and idle() restarts.
 if(idle()) // returns 1 if any new key is available
 {
   if(newKey)
   {
     if(newKey == CTR)
     {
       menu(); // Lets us set temps and humidity
     }
   }
 }
} 

void checkSensors()
{
  int chk0 = sensor0.read();
  
  if(chk0 != 0){ //if 0 show error
    #ifdef DEBUG
    Serial.print("sensor0 error: ");
    switch (chk0)
    {
      case -1: Serial.println("Checksum"); break;
      case -2: Serial.println("Time out"); break;
      default: Serial.println("Unknown"); break;
    }
    #endif    
  }
  else{ // no error    
    tempOutdoor = sensor0.fahrenheit();
    humidityOutdoor = sensor0.humidity;
  }

  int chk1 = sensor1.read();
  
  if(chk1 != 0){ // if 0 show error
    #ifdef DEBUG
    Serial.print("sensor1 error: ");
    switch (chk1)
    {
      case -1: Serial.println("Checksum"); break;
      case -2: Serial.println("Time out"); break;
      default: Serial.println("Unknown"); break;
    } 
    #endif   
  }
  else{ // No error so show data    
    tempIndoor = sensor1.fahrenheit();
    humidityIndoor = sensor1.humidity;
  }
  
  // Run the algorithm
  if(humidityOutdoor >= setHumidity)
  {
    #ifdef DEBUG
    Serial.println("humidityOutdoor >= setHumidity");
    #endif
    fanState = 0;
  }
  else if( (tempOutdoor > setTempLow) && ( tempOutdoor < tempIndoor) && (setTempLow != -1))
  {
    #ifdef DEBUG  
    Serial.println("(tempOutdoor > setTempLow) && ( tempOutdoor < tempIndoor) && (setTempLow != -1)");
    #endif
    fanState = 1;    
  }
  else if( (tempOutdoor < setTempHigh) && ( tempOutdoor > tempIndoor) && (setTempHigh != -1))
  {
    #ifdef DEBUG
    Serial.println("(tempOutdoor < setTempHigh) && ( tempOutdoor > tempIndoor) && (setTempHigh != -1)");
    #endif
    fanState = 1;    
  } 
  else 
  {
    #ifdef DEBUG
    Serial.println("None of the above! Fan off");
    #endif
    fanState = 0;    
  } 
  
  #ifdef DEBUG  
  Serial.print("Fan state = ");
  Serial.println(fanState);
  #endif
  
  setFan(fanState);
}

// Turn the fan on or off
void setFan(uint8_t state)
{
  if(fanState == 1)// turn it on
  {
    digitalWrite(fanPin,HIGH); // Set pin to Vcc
  }
  else // turn it off
  {
    digitalWrite(fanPin,LOW); // Set pin to ground   
  }  
}


// Program memory strings
p_string(setTHigh) = "Set TH?";
p_string(setTLow) = "Set TL?";
p_string(setHumid) = "Set Hum?";
p_string(yes) = "<Yes Xt>";

// Used by showSelection
#define SETTH 0
#define SETTL 1
#define SETHUM 2
#define SELECTIONS 2
uint8_t selection = 0;

uint8_t inMenu = 1;

void menu()
{
  
  #ifdef DEBUG
  Serial.println("menu()");
  #endif 
  
  uint8_t key = newKey;
  
  mt_print("Move Key",0,0);
  mt_print("Up or Dn",0,1); 
  delay(500);
  showSelection(0);
  selection = 0; 
  
  while(inMenu)
  {
    if(checkKeyDelay(250))
    {
      checkKey = 0;
      if(newKey)
      {
        if(newKey == UP)// Scroll up through menu
        {
          if(selection > 0)selection--;
          showSelection(selection); 
        }
        else if(newKey == DWN)// Schroll down thru menu
        {
          if(selection < SELECTIONS) selection++;
          showSelection(selection); 
        }
        else if(newKey == LFT)// Yes, select the current menu item.
        {
          if(selection == 0) doSetTempHigh();
          else if (selection == 1)doSetTempLow();
          else if (selection == 2)doSetHumidity();
          else // something is wrong
          {
             selection = 0; 
             break;
          } 
          showSelection(selection);
        }
        else if(newKey == RGT) // exit
        {
          selection = 0;
          break;        
        }     
        newKey = 0;
      }
    } 
  }  
}


void showSelection(uint8_t selected)
{
  #ifdef DEBUG
  Serial.print("selected: ");
  Serial.println(selected);
  #endif
  
  // Show selected query
  if(selected == SETTH)
  {
    mt_printPM(setTHigh,0,0);   
  }
  else if (selected == SETTL)
  {
    mt_printPM(setTLow,0,0);   
  }
  else if (selected == SETHUM)
  {
    mt_printPM(setHumid,0,0); 
  }
  else // something is wrong
  {
    mt_print("????????",0,0);
    delay(1000);
    return;
  } 
  
  // See if the user wants to set this value
  mt_printPM(yes,0,1);  
}

void doSetTempHigh()
{ 
  /*mt_print("In setTH",0,0);
  mt_print("Dosmthng",0,1);
  delay(500); 
  */
  setTempHigh = mt_get8BitNumber("THigh:",0);
  
  mt_print("STH got:",0,0);
  mt_print8BitNumber(setTempHigh,0,1);
  delay(1000);
}

void doSetTempLow()
{ 
  setTempLow = mt_get8BitNumber("TLow:",0);
  
  mt_print("STL got:",0,0);
  mt_print8BitNumber(setTempLow,0,1);
  delay(1000); 
}

void doSetHumidity()
{ 
  setHumidity = mt_get8BitNumber("Humid:",0);
  
  mt_print("SHU got:",0,0);
  mt_print8BitNumber(setHumidity,0,1);
  delay(1000);
}

// Show the existing data and wait for a keypress
// void return if a key is pressed
#define IDLEDELAY 1000
uint8_t idle()
{ 
    mt_printPM(tempOut,0,0);
    mt_print8BitNumber(tempOutdoor,0,1);
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(tempIn,0,0);
    mt_print8BitNumber(tempIndoor,0,1);
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(humOut,0,0);
    mt_print8BitNumber(humidityOutdoor,0,1);    
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(humIn,0,0);
    mt_print8BitNumber(humidityIndoor,0,1);    
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(tempHighSet,0,0);
    mt_print8BitNumber(setTempHigh,0,1);    
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(tempLowSet,0,0);
    mt_print8BitNumber(setTempLow,0,1);
    if(checkKeyDelay(IDLEDELAY)) return(1);
    mt_printPM(humiditySet,0,0);
    mt_print8BitNumber(setHumidity,0,1);    
    if(checkKeyDelay(IDLEDELAY)) return(1);
    if(idleState < 6) idleState++;
    else idleState = 0;
    return(0);
}

// This is not exactly a millisecond delay
// since some time gets wasted looking for
// the key and other overhead.
uint8_t checkKeyDelay(uint16_t milli)
{
  
 checkSensors();  
  
 if(!milli)milli = 1;
 while(milli--)
 {
   // check key
   if (checkKey)
   {
     checkKeys();
     checkKey = 0;
     if(gotNewKey) return(1); // bail out of delay
   }
  // delay 1 ms
  delay(1); 
 }
 return (0); // no new key
}

void checkKeyTimer()
{
  checkKey = 1;  
}

// Rather than use the loop to check the keys continuously
// and thereby adding quite a load on the processor, we will
// use a timer set to check the keys 10 times per second
void checkKeys()
{  
  newKey = mt_getKey();
  if(newKey)// not 0 if a new key is found
  {
    gotNewKey = 1; 

  }
  else gotNewKey = 0;
}


